/*
** Module   : HtmlResourceHandler.java
** Abstract : delivers main page to AJAX clients
**
** Copyright (c) 2009-2013, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**         Golden Code Development Corporation
**                 CONFIDENTIAL
**
** -#- -I- --Date-- ----------------------Description--------------------------
** 001 MAG 20131105 First version
*/

package com.goldencode.p2j.ws;

import java.io.*;
import java.util.logging.*;

import javax.servlet.*;
import javax.servlet.http.*;

import org.eclipse.jetty.http.*;
import org.eclipse.jetty.server.handler.*;
import org.eclipse.jetty.server.*;

import com.goldencode.p2j.util.*;

/**
 * Jetty Handler which delivers HTML resource files.
 * This class provide a method to parse the HTML text and
 * substitute place holders. 
 */
public class HtmlResourceHandler
extends AbstractHandler
{
   /** Logger. */
   private static final Logger LOG = 
            LogHelper.getLogger(HtmlResourceHandler.class.getName());

   /** The root portion of the target request name (part of the URL). */
   private String targetRoot = null;
   
   /** The html skeleton page. */
   private String skeletonPage = null;
     
   /**
    * Constructor.
    *
    * @param    targetRoot
    *           Prefix for all URLs relating to this page.
    * @param    skeletonPage
    *           Fully qualified resource name of the HTML file which should be used.
    */
   public HtmlResourceHandler(String targetRoot,
                              String skeletonPage)
   {
      this.targetRoot   = targetRoot;
      this.skeletonPage = skeletonPage;  
   }
   
   /**
    * Return the main page for AJAX clients.
    *    
    * @param    target 
    *           The target of the request - either a URI or a name.
    * @param    base
    *           The base request.
    * @param    request
    *           The object or a wrapper of the request.
    * @param    response 
    *           The object or a wrapper of the response. 
    */
   public void handle(String              target, 
                      Request             base,
                      HttpServletRequest  request, 
                      HttpServletResponse response)
   throws IOException, 
          ServletException
   {
      if (!base.isHandled() && target.equalsIgnoreCase(targetRoot))
      {
         generateMainPage(base, response);
      }     
   }
      
   /**
    * Use the skeleton main page to generate a response.
    *
    * @param    base
    *           The details and state the request.
    * @param    response 
    *           The details and state of the response. 
    */
   private void generateMainPage(Request base, 
                                 HttpServletResponse response)
   throws IOException, 
          ServletException
   {
      // produce the page contents
      response.setContentType(MimeTypes.Type.TEXT_HTML.asString());
      response.setStatus(HttpServletResponse.SC_OK);
      
      PrintWriter writer = response.getWriter();
      
      try
      {
         InputStream is = this.getClass().getResourceAsStream(skeletonPage);
         BufferedReader reader = new BufferedReader(new InputStreamReader(is));
         
         String str = null;
         
         // read the skeleton page and translate %substitutions
         // on the fly
         while (null != (str = reader.readLine()))
         {
            writer.println(handleReplacements(base, str));
         }
         
         reader.close();
      }
      catch (IOException ioe)
      {
         LOG.logp(Level.SEVERE,
                  "HtmlResourceHandler.generateMainPage()",
                  "",
                  LogHelper.generate("IOException!"),
                  ioe);
      }
      finally
      {
         // mark the request as handled
         base.setHandled(true);
      }
   }
      
   /**
    * Process the given string and make any replacements of parameter values
    * as needed. The replacement placeholder follow the JEE expression
    * language (EL)format for immediate evaluation format. 
    * Example: ${variable}
    * Extends this class and override this method to provide your custom code
    * for place holders replacement.   
    *
    * @param    base
    *           The details and state the request.
    * @param    text
    *           The text line that may need modifications.
    *
    * @return   The new text if a modification occurred or the original text
    *           if no modification was needed.
    */
    protected String handleReplacements(Request base, String text)
    {
       return text;
    }
         
}
