/*
 ** Module   : WsServer.java
 ** Abstract : provide AJAX style services using Web Sockets
 **
 ** Copyright (c) 2009-2013, Golden Code Development Corporation.
 ** ALL RIGHTS RESERVED. Use is subject to license terms.
 **
 **         Golden Code Development Corporation
 **                 CONFIDENTIAL
 **
 ** -#- -I- --Date-- ----------------------Description--------------------------
 ** 001 MAG 20131105 First version based on jetty 9.1 web sockets
 */

package com.goldencode.p2j.ws;

import java.io.*;
import java.util.logging.*;

import javax.net.ssl.*;
import javax.servlet.*;
import javax.servlet.http.*;

import org.eclipse.jetty.http.*;
import org.eclipse.jetty.server.*;
import org.eclipse.jetty.server.Handler;
import org.eclipse.jetty.server.Server;
import org.eclipse.jetty.server.handler.*;
import org.eclipse.jetty.util.ssl.*;
import org.eclipse.jetty.util.thread.*;
import org.eclipse.jetty.websocket.server.*;
import org.eclipse.jetty.websocket.servlet.*;

import com.goldencode.p2j.cfg.*;
import com.goldencode.p2j.security.SecurityManager;
import com.goldencode.p2j.util.*;
import com.goldencode.p2j.ws.test.*;

/**
 * Web Sockets jetty embedded server implementation.
 * 
 * @author mag
 *
 */
public class WsServer
{

   /** Logger. */
   private static final Logger LOG = LogHelper
            .getLogger(TestPageHandler.class.getName());
   
   /** The bootstrap configuration */
   private BootstrapConfig config;

   /** Embedded jetty server */
   private Server server;

   /** Context Handlers */
   private HandlerCollection handlerCollection = new HandlerCollection();

   /** Context path */
   private String contextPath = "/";

   /** Server port */
   private int port;

   /** Server host name */
   private String host;

   /**
    * Startup the embedded server.
    * The sever listen on any available address.
    * The port number is dynamically allocated by the OS.
    * 
    * @throws  Exception 
    *          If an error condition occur during the server startup. 
    */
   public void startup() 
   throws Exception
   {
      this.startup(null, 0);
   }

   /**
    * Startup the embedded server.
    * The port number is dynamically allocated by OS.
    * 
    * @param   host
    *          Host name or IP. If <b>null</b> server listen on all addresses.
    * 
    * @throws  Exception 
    *          If an error condition occur during the server startup. 
    */
   public void startup(String host) 
   throws Exception
   {
      this.startup(host, 0);
   }
   
   /**
    * Startup the embedded server.
    * 
    * @param   host
    *          Host name or IP. If <b>null</b> server listen on all addresses.
    * @param   port
    *          Port number to listen. If <b>0</b> the port is OS dynamically allocated.
    * 
    * @throws  Exception 
    *          If an error condition occur during the server startup. 
    */
   public void startup(String host, int port) 
   throws Exception
   {
      // SSL Context Factory for HTTPS
      SslContextFactory sslContextFactory = getSslContextFactory(config);

      QueuedThreadPool pool = new QueuedThreadPool();
      pool.setDaemon(true);

      server = new Server(pool);
      server.manage(pool);
      server.addConnector(configSslConnector(server, 
                                             host, 
                                             port,
                                             sslContextFactory));
      server.setHandler(configWsContext());
      server.setStopAtShutdown(true);
      server.start();

      // get host and port after server is starting
      getRuntimeParams();
      
      LOG.logp(Level.INFO,
               "WsServer.startup()",
               "",
               LogHelper.generate("Server URL: https://%s:%d%s", 
                        host, port, contextPath));      
      
   }

   /**
    * Shutdown server
    * 
    * @throws Exception
    *         If the server cannot be stopped.
    */
   public void shutdown() 
   throws Exception
   {
      if (server != null && server.isRunning())
      {
         server.stop();
      }
   }
   
   /**
    * Set the bootstrap configuration structure if any.
    * 
    * @param config
    */
   public void setBootstrapConfig(BootstrapConfig config)
   {
      this.config = config;
   }

   /**
    * Get the server host name and port after the server
    * is started and running. 
    *  
    */
   private void getRuntimeParams()
   {
      if(server.isRunning())
      {
         Connector[] connectors = server.getConnectors();
         ServerConnector connector = (ServerConnector) connectors[0];
         port = connector.getLocalPort();
         host = connector.getHost();
      }
   }

   /**
    * Register a new handler on this server context
    * 
    * @param handler
    *        A Handler instance.
    */
   public void addHandler(Handler handler)
   {
      handlerCollection.addHandler(handler);
   }

   /**
    * Register a new web socket.
    * 
    * @param   tag
    *          The web socket target 
    * @param   webSocket
    *          A POJO class decorated with @WebSocket annotation
    */
   public void addWsHandler(String target, Class<?> webSocket)
   {
      Handler wsHandler = new WsServer.WsHandler(target, webSocket);
      handlerCollection.addHandler(wsHandler);
   }

   /**
    * Construct a SSL Context Factory for HTTPS
    * 
    * @param   config
    *          The client bootstrap configuration. 
    * @return  The SslContextFactory instance          
    * 
    * @throws  Exception
    *          If any error conditions occur.
    */
   private SslContextFactory getSslContextFactory(BootstrapConfig config) 
   throws Exception
   {      
      SecurityManager sm = SecurityManager.getInstance();
      SslContextFactory sslContextFactory = new SslContextFactory();
      SSLContext sslContext = 
               config == null ? sm.getSecureSocketContext() :
                                sm.getSecureSocketContext(config);
      sslContextFactory.setSslContext(sslContext);
      
      return sslContextFactory;      
   }
   
   /**
    * Create and configure a SSL server connector.
    *   
    * @param   server
    *          The server instance.
    * @param   host
    *          The server host name or IP address.
    * @param   port
    *          The port number to listen.
    * @param   sslContextFactory
    *          A SslContextFactory instance.
    *          
    * @return  The SSL server connector
    */
   private ServerConnector configSslConnector(Server server,
                                           String host, 
                                           int port, 
                                           SslContextFactory sslContextFactory)
   {
      String protocol = HttpVersion.HTTP_1_1.asString();
      ServerConnector sslConnector = new ServerConnector(server,
         new SslConnectionFactory(sslContextFactory, protocol),
         new HttpConnectionFactory(new HttpConfiguration()));
      sslConnector.setHost(host);
      sslConnector.setPort(port);
      
      return sslConnector;
   }

   /**
    * Create and configure a ContextHandler for this server
    * 
    * @return  The ContextHandler used as server Handler
    */
   private Handler configWsContext()
   {
      ContextHandler context = new ContextHandler();
      context.setContextPath(contextPath);
      context.setResourceBase(".");
      context.setClassLoader(Thread.currentThread().getContextClassLoader());
      context.setHandler(handlerCollection);
      
      return context;
   }

   /**
    * Set the context path
    * 
    * @param   contextPath 
    *          A string representing the context path root.
    */
   public void setContextPath(String contextPath)
   {
      this.contextPath = contextPath;
   }

   /**
    * Get runtime port number.
    * 
    * @return  The port number on which the server listen to.
    */
   public int getPort()
   {
      return port;
   }

   /**
    * Return the runtime host name.
    * 
    * @return  The host name known after the server is running.
    *
    */
   public String getHost()
   {
      return host == null ? "localhost" : host;
   }
   
   /**
    * Register and handle web sockets.
    * This class is used to create and register a jetty 
    * Handler for each web socket.
    * 
    */
   private static class WsHandler extends WebSocketHandler
   {
      private Class<?> wsClass;
      private String   wsTarget;

      /**
       * Constructor
       * 
       * @param   wsTarget 
       *          WebSocket target string.
       * 
       * @param   wsClass
       *          WebSocket class.
       */
      public WsHandler(String wsTarget, Class<?> wsClass)
      {
         this.wsTarget  = wsTarget;
         this.wsClass = wsClass;
      }

      @Override
      public void configure(WebSocketServletFactory wsServletFactory)
      {
         wsServletFactory.register(wsClass);
      }

      @Override
      public void handle(String target, 
                         Request baseRequest,
                         HttpServletRequest request, 
                         HttpServletResponse response)
      throws IOException, ServletException
      {
         if (!baseRequest.isHandled() && wsTarget.equals(target))
         {
            super.handle(target, baseRequest, request, response);
            baseRequest.setHandled(true);
         }
      }
   }

}
